--=====================================================================================
-- SERVICE TRUCK
--
-- @For:		Anybody
-- @Date:		2017-10-14
-- @Purpose:	To affix vehicle selling trigger to truck, enabling field repairs.
--=====================================================================================


Service = {};

function Service.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Drivable, specializations)
end;

function Service:load(savegame)
	self.addVehicleSellTrigger = Service.addVehicleSellTrigger
	self.removeVehicleSellTrigger = Service.removeVehicleSellTrigger
	
	self.vehicleSellTriggers = {}
	
	local vehicleSellTriggerNode = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.vehicleSellTrigger#index"))

	if vehicleSellTriggerNode ~= nil then
		self.vehicleSellTrigger = VehicleSellingPoint:new(vehicleSellTriggerNode, self)
	end;
	
end;

function Service:delete()
	for _, trigger in pairs(self.vehicleSellTriggers) do
		trigger:onVehicleDeleted(self);
	end;

	if self.vehicleSellTrigger ~= nil then
		self.vehicleSellTrigger:delete();
		self.vehicleSellTrigger = nil;
	end;
end;

function Service:readStream(streamId, connection)
end;

function Service:writeStream(streamId, connection)
end;

function Service:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Service:keyEvent(unicode, sym, modifier, isDown)
end;

function Service:update(dt)
end;

function Service:updateTick(dt)
end;

function Service:draw()	
end;

function Service:addVehicleSellTrigger(trigger)
	table.insert(self.vehicleSellTriggers, trigger);
end;

function Service:removeVehicleSellTrigger(trigger)
	for i = 1, table.getn(self.vehicleSellTriggers) do
		if self.vehicleSellTriggers[i] == trigger then
			table.remove(self.vehicleSellTriggers, i);
			break;
		end;
	end;
end;

oldVehicleSellingPointNew = VehicleSellingPoint.new
VehicleSellingPoint.new = function(self, id)

	local self
	if oldVehicleSellingPointNew ~= nil then
		self = oldVehicleSellingPointNew(self, id)
	end;

	self.vehiclesSellTriggerCount = {}
	
	return self
end;

oldVehicleSellingPointDelete = VehicleSellingPoint.delete
VehicleSellingPoint.delete = function(self)
	
	oldVehicleSellingPointDelete(self)
	
	for vehicle, count in pairs(self.vehiclesSellTriggerCount) do
		if count > 0 then
			if vehicle.removeVehicleSellTrigger ~= nil then
				vehicle:removeVehicleSellTrigger(self);
			end;
		end;
	end;
	
end;

function VehicleSellingPoint:onVehicleDeleted(vehicle)
	self.vehiclesSellTriggerCount[vehicle] = nil
end;

VehicleSellingPoint.triggerCallback = function(self, triggerId, otherId, onEnter, onLeave, onStay)
	if self.isEnabled and (not g_isPresentationVersion or g_isPresentationVersionShopEnabled) and g_currentMission.missionInfo:isa(FSCareerMissionInfo) then

		if onEnter or onLeave then
			if g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
				if onEnter then
					if not self.objectActivated then
						g_currentMission:addActivatableObject(self)
						self.objectActivated = true
					end;
				elseif onLeave then
					if self.objectActivated then
						g_currentMission:removeActivatableObject(self)
						self.objectActivated = false
					end;
				end;
			end;
			
			local vehicle = g_currentMission.nodeToVehicle[otherId];
			if vehicle ~= nil then 
				if vehicle.addVehicleSellTrigger ~= nil and vehicle.removeVehicleSellTrigger ~= nil and vehicle ~= self then
					local count = Utils.getNoNil(self.vehiclesSellTriggerCount[vehicle], 0);
					if onEnter then
						self.vehiclesSellTriggerCount[vehicle] = count + 1
						if count == 0 then
							vehicle:addVehicleSellTrigger(self);
						end;
					else -- onLeave
						self.vehiclesSellTriggerCount[vehicle] = count - 1
						if count == 1 then
							self.vehiclesSellTriggerCount[vehicle] = nil;
							vehicle:removeVehicleSellTrigger(self);
						end;
					end;
				end;
			end;
		end;
	end;
end;

VehicleSellingPoint.sellAreaTriggerCallback = function(self, triggerId, otherId, onEnter, onLeave, onStay, otherShapeId)
	if otherShapeId ~= nil and (onEnter or onLeave) then
		if onEnter then
			self.vehicleInRange[otherShapeId] = true
		elseif onLeave then
			self.vehicleInRange[otherShapeId] = nil
		end;

		self:determineCurrentVehicle()
		
		local vehicle = self.currentVehicle
		if vehicle ~= nil then
			if onEnter then
				vehicle.rvWIRTrigger = true
			else
				vehicle.rvWIRTrigger = false
			end;
		end;

	end;
end;

Mission00.loadMission00Finished = Utils.appendedFunction(Mission00.loadMission00Finished, function(...)
	local repairMod = "FS17_repairVehicles"

	if g_modIsLoaded[repairMod] then
		envRepairVehicles = getfenv(0)[repairMod]
	end;
	
	if envRepairVehicles ~= nil then
		envRepairVehicles.repairVehicles.updateTick = Utils.overwrittenFunction(envRepairVehicles.repairVehicles.updateTick, function(self, dt)
			if g_currentMission.player ~= nil then
				local vx, vy, vz = getWorldTranslation(g_currentMission.player.rootNode);
				local sx, sy, sz = getWorldTranslation(self.rootNode); 
				local distance = Utils.vector3Length(sx-vx, sy-vy, sz-vz);
				if distance < 3.5 then
					self.rvPIR = true; 
				else
					self.rvPIR = false; 
				end;
			end;

			if self.vehicleNode == nil then
				for vehicleNode,vehicleTable in pairs(g_currentMission.nodeToVehicle) do
					if vehicleTable == self then
						self.vehicleNode = vehicleNode;
					end
				end
			else
				local numWorkshops = 0;
				self.rvWIR = false;
				for infoTable,name in pairs(g_currentMission.objectsToClassName) do
					if name == "Workshop" then
						numWorkshops = numWorkshops + 1;
						if infoTable.sellingPoint.vehicleInRange ~= nil then
							if infoTable.sellingPoint.vehicleInRange[self.vehicleNode] ~= nil then
								self.rvWIR = true;
								break;
							end
						end
					end
				end
				if numWorkshops == 0 then
					self.rvWIR = false;
				end
				if self.rvWIRTrigger and not self.rvWIR then
					self.rvWIR = true
				end;
			end;
		end)
	end;
end)